#
# spec file for package s3backer
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


# Should we try to include NDB support?
%if 0%{?sle_version} >= 150300
%define         include_nbd 1
%define         runfilesdir /run
%define         nbdsockdir  %{runfilesdir}/s3backer-nbd
%define         tmpfileconf s3backer-nbd.conf
%endif

Name:           s3backer
Version:        2.1.5
Release:        1.1
Summary:        FUSE and NBD single file backing store via Amazon S3
License:        GPL-2.0-or-later
Group:          System/Filesystems
URL:            https://github.com/archiecobbs/%{name}
Source:         https://s3.amazonaws.com/archie-public/%{name}/%{name}-%{version}.tar.gz
BuildRequires:  fuse3-devel >= 3.0.0
BuildRequires:  libcurl-devel >= 7.16.2
BuildRequires:  libexpat-devel
BuildRequires:  libopenssl-devel
%if 0%{?include_nbd}
BuildRequires:  libzstd-devel
%endif
%if 0%{?include_nbd}
BuildRequires:  nbd
BuildRequires:  nbdkit-devel
BuildRequires:  nbdkit-server
Requires:       nbd
Requires:       nbdkit-server
%endif
BuildRequires:  pkgconfig
BuildRequires:  zlib-devel

%description
s3backer is a filesystem that contains a single file backed by the Amazon
Simple Storage Service (Amazon S3).  As a filesystem, it is very simple:
it provides a single normal file having a fixed size.  Underneath, the
file is divided up into blocks, and the content of each block is stored
in a unique Amazon S3 object.  In other words, what s3backer provides is
really more like an S3-backed virtual hard disk device, rather than a
filesystem.

In typical usage, a `normal' filesystem is mounted on top of the file
exported by the s3backer filesystem using a loopback mount (or disk image
mount on Mac OS X).

s3backer can also function as a Network Block Device (NBD) plug-in.

%prep
%setup -q

%build
export SUSE_ASNEEDED=0

# This is needed so /usr/sbin/nbdkit, etc. will be found by AC_PATH_PROGS
export PATH="${PATH}:%{_sbindir}"

%configure
%make_build

%install
make install DESTDIR='%{buildroot}'
install -m 0644 COPYING %{buildroot}%{_docdir}/%{name}/
rm -f %{buildroot}%{_docdir}/%{name}/INSTALL

%if 0%{?include_nbd}
mkdir -p %{buildroot}%{_tmpfilesdir}
cat > %{buildroot}%{_tmpfilesdir}/%{tmpfileconf} << 'xxxEOFxxx'
# See tmpfiles.d(5) for details
d %{nbdsockdir} 0700 root root -
xxxEOFxxx
%endif

%if 0%{?include_nbd}
%post
%tmpfiles_create %{_tmpfilesdir}/%{tmpfileconf}
%endif

%files
%{_bindir}/*
%{_mandir}/man1/*
%if 0%{?include_nbd}
%{_libdir}/nbdkit/plugins/*
%{_tmpfilesdir}/%{tmpfileconf}
%ghost %dir %attr(0700,root,root) %{nbdsockdir}
%endif
%{_docdir}/%{name}

%changelog
* Thu May 22 2025 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.1.5
  - Add new flag "--statsFileMirror" (issue #237)
  - Update to FUSE 3.x (#239)
* Sat May 10 2025 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.1.4
  - Add new flag "--sharedDiskMode" (issue #236)
  - Fix use of deprecated cURL constant (pr #228)
  - Conditionalize nbdkit "block_size" callback (issue #226)
  - Skip nbd-client(8) cleanup step if it failed to start properly (#227).
  - Add new flag "--accessEC2IAM-IMDSv2" (issue #228)
* Wed Jun 12 2024 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.1.3
  - Implement nbdkit "block_size" callback
  - Added configure flag "--enable-nbd" (issue #223)
  - Fix spurious "cache file is truncated" error (issue #222)
  - Fix bugs in dcache.c when USE_FALLOCATE enabled (issue #224)
  - Automatically recreate nbdkit socket's directory if missing (issue #219)
  - Recalculate date and authorization on retry (issue #214)
* Mon Oct 23 2023 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.1.1
  + Fixed bug with bulk delete HTTP requests (issue #211)
  + Fixed empty payload bug when retrying an upload (issue #212)
  + Check for errors returned by curl_easy_setopt()
  + Added --noCurlCache flag to facilitate debugging
  + Bump wait time for nbdkit to start from 1s to 5s
* Sat Sep 23 2023 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.1.0
  + Avoid sending an empty Accept-Encoding header (issue #208)
  + Use fallocate(FALLOC_FL_PUNCH_HOLE) for empty disk cache blocks (issue #200)
  + Made sync(1) work on s3backer file to flush cached data (issue #197)
  + Fixed bug where s3b_compress=deflate NDB flag would fail (issue #195)
  + Fixed bug in zero cache "current_cache_size" stats value (issue #204)
  + Reuse cURL handles after normal HTTP error codes (issue #199)
  + Run modprobe(8) if needed when starting with the --nbd flag (issue #203)
  + Use newer OpenSSL 3.0 API functions when available
* Sat Jul 30 2022 Archie Cobbs <archie.cobbs@gmail.com>
- Don't try to build with NBD until 15.3 or later
* Fri Jul 29 2022 Archie Cobbs <archie.cobbs@gmail.com>
- Fix bug in spec file (was missing /run/s3backer-ndb)
* Sun Jul 17 2022 Archie Cobbs <archie.cobbs@gmail.com>
- Fix build on older systems that don't have libzstd.
* Sun Jul 17 2022 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.0.2
  + Fixed bugs with the --nbd flag when not also using -f flag (issue #188)
  + Fixed free() of invalid pointer bug in zero_cache.c (issue #191)
  + Added support for zstd compression (pr #187)
* Sat Jun  4 2022 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.0.1
  + Fix null pointer dereference bug in zero cache.
* Sat Jun  4 2022 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 2.0.0
  + Support running as a Network Block Device (NBD) server (issue #178)
  + Fix slow write performance with large block sizes since 1.6.0 (issue ##185)
  + Disable MD5 cache by default now that Amazon S3 is consistent (issue #183)
  + Fix bug where "(null)" was appearing in usage message (issue #161)
  + Give more meaningful usage error when size limits are exceeded (issue #162)
  + Added flag "--http11" to restrict cURL to HTTP 1.1 (issue #168)
  + Clean up mount token if FUSE itself fails to start (issue #175)
  + Detect HTTP redirects and emit appropriate hint/warning (issue #174)
  + Added man page reference to Linux bcache (issue #169)
  + Added "--blockCacheFileAdvise" flag (pr #176)
  + Treat 3xx HTTP response codes as errors
* Wed Oct 27 2021 Archie Cobbs <archie.cobbs@gmail.com>
- Run spec-cleaner on spec file
* Sat Oct  2 2021 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 1.6.3
  + Fixed bug with `--listBlocks' threads doing redundant overlapping queries
  + Refactor to support multiple different compression algorithms
  + Fixed mutex handling bug in block cache (issue #152)
  + Release mutexs prior to destruction (issue #151)
  + Fixed use-after-free bug in XML parser (pr #154)
* Mon Jun 14 2021 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 1.6.2
  + Require test directory to be absolute unless `-f' flag given (issue #143)
  + Show HTTP error response payload content when `--debug-http' flag given
  + List blocks in the background; added `--listBlocksThreads' (issue #24)
  + Don't let IAM credentials expire while listing blocks (issue #146)
  + Fixed bug parsing "--configFile" inside comma-separated option list
* Wed Dec 30 2020 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 1.6.1
  + Fixed integer overflow bug setting "x-amz-meta-s3backer-filesize" (issue #141)
* Wed Dec 23 2020 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 1.6.0
  + Added `--sse-key-id' flag (issue #137)
  + Log a more useful error message when IAM credentials not found (issue #136)
  + Warn on startup if disk space is insufficient for disk cache (issue #138)
  + Added zero block cache to better handle fstrim operations (issue #139)
* Fri Oct 30 2020 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 1.5.6
  + Fixed phantom write error when server-side-encryption used (issue #135)
  + Allow bucket names with "subdirectory" for prefix (issue #130)
  + Added `--configFile' flag (issue #129)
  + Added `--accessKeyEnv' flag (issue #128)
  + Removed deprecated `--rrs' flag
* Sat Aug 22 2020 Archie Cobbs <archie.cobbs@gmail.com>
- Upgrade to release 1.5.5
  + Added `--no-vhost' flag (issue #117)
  + Added `--blockCacheNumProtected' flag (pr #119)
  + Added `--test-errors', `--test-delays', and `--test-discard'
  + Disallow stream encryption ciphers (issue #123)
* Tue Oct  8 2019 <archie.cobbs@gmail.com>
- Upgrade to release 1.5.4
  + Only set "x-amz-server-side-encryption" header with PUT requests (issue #116)
  + Don't kill IAM thread unless actually started (issue #115).
* Fri Aug  9 2019 <archie.cobbs@gmail.com>
- Update to release 1.5.3
  + Fixed bug where IAM update thread was killed after fork (issue #115)
  + Fixed use-after-free bug in block_cache_verified() (issue #113)
  + Fixed use-after-free bug when updating IAM credentials (pr #114)
  + Fixed bug in test mode that was causing bogus I/O errors
* Tue Jul  9 2019 <archie.cobbs@gmail.com>
- Update to release 1.5.2
  + Fixed bug where block cache would not work when run in the background (issue #112)
  + Fixed bug where we were not parsing HTTP headers case-insensitively (pr #11)
  + Bail out during `--listBlocks' if we see an object name past our block range
  + Added `--blockHashPrefix' flag (issue #80)
* Mon Apr 15 2019 archie.cobbs@gmail.com
- Update to release 1.5.1
  + Fixed a few places where fixed-sized buffers were too small (issue #108)
  + Don't claim cache hit if partial write required reading the block (pr #103)
  + Exit process with error code if s3backer store setup fails at startup
  + Reset statistics if stats file is unlinked (issue #106)
* Sat Jun  9 2018 archie.cobbs@gmail.com
- Update to release 1.5.0
  + Add support for recovering dirty blocks in the disk cache (issue #87)
  + Replaced boolean 'mounted' flag with a unique 32-bit mount token (issue #87)
  + Wait for min_write_delay before access after write error (issue #76)
  + Configure TCP keep-alive on HTTP connections (issue #78)
  + Added support for server side encryption (pull #81)
* Wed Feb  1 2017 archie.cobbs@gmail.com
- Update to release 1.4.4
  + Added `--defaultContentEncoding' for non-compliant backends (issue #68)
  + Fixed auth bug when prefix contains URL-encodable char (issue #69)
  + Remove restriction preventing streaming encryption modes (issue #70)
* Mon Jul 25 2016 archie.cobbs@gmail.com
- Update to release 1.4.3
  + Add support for STANDARD_IA storage class (issue #59)
  + Set "Accept-Encoding" header appropriately (issue #48)
  + Fix build issue with OpenSSL 1.1.0 (issue #64)
* Tue Sep  1 2015 archie@dellroad.org
- Update to release 1.4.2
  + Update license to fix OpenSSL vs. GPL conflict
* Mon May  4 2015 archie@dellroad.org
- Update to release 1.4.1
  + Fix use-after-free bug configuring base URL (github issue #44)
* Fri Apr 17 2015 archie@dellroad.org
- Update to release 1.4.0
  + Added support for authentication version 4 (issue #51)
  + Added support for credentials via IAM role from EC2 meta-data (issue #48)
  + Fixed bug where `--erase' did not clear the mounted flag
  + Moved from Google project hosting to GitHub
  + Fixed compile problem on FreeBSD
* Thu Jul 18 2013 archie@dellroad.org
- Update to version 1.3.7
  + Add `--keyLength' for overriding generated encryption key length
* Tue Jul 16 2013 archie@dellroad.org
- Update to version 1.3.6
  + Fix use of MAX_HOST_NAME in http_io.c (issue #42)
  + Fix encryption key generation bug (on some systems)
* Wed May 29 2013 archie@dellroad.org
- Update to version 1.3.5
  + Check for duplicate mount at startup (issue #10)
  + Minor spec file cleanups
* Wed Apr  3 2013 archie@dellroad.org
- Update to version 1.3.4
  + Add support for fallocate(2)
* Wed Jan 23 2013 archie@dellroad.org
- Fix License to use SPDX format
- Install COPYING into docs
- Remove INSTALL from docs
* Sat Apr  7 2012 archie@dellroad.org
- Update to version 1.3.3
* Sat May 14 2011 archie@dellroad.org
- Update to version 1.3.2
* Sun Jul 18 2010 archie@dellroad.org
- Added SUSE_ASNEEDED magic to fix build
